# Development

## General

1. Create branch - follow the branching guidelines: `git checkout -b user/[username]/[branch-name]`
2. Make and commit your changes
    - If you're creating a **NEW** component in src/components, you'll need to do the following 3 things:
      1. Add it to the public component interface (src/main.js) 
      2. Add to the type definition file (types/templates/module-declaration.d.ts). This makes consumers aware of the component and its types.
      3. Add an entry for the CSS (src/oui/_oui-partials.scss)
3. Update `CHANGELOG.md` and note your changes directly under "Unreleased":

   ```md
   ## Unreleased

   - [Release/Feature/Patch] Describe your change here. Component names are in **bold** and prop names are in `monospace` ([#GITHUB_ISSUE](Link to github issue))
   ```

   > **Note:** please do not create a new version number header, this is done only during release.

5. Regenerate Typescript definitions by running `yarn generate-types` (for React component changes)
6. `git push` your changes to GitHub
7. [Open a pull request](https://github.com/optimizely/axiom/compare) of your branch, add at least one reviewer
8. Review the results of Chromatic's visual regression tests. See below for [more info about VRT](#-using-chromatic-visual-regression-testing-vrt)
9. Once you've merged your PR, please follow the instructions to [Release a New Version](https://github.com/optimizely/axiom/blob/devel/CONTRIBUTING.md#ship-release-a-new-version), including updating the version in the monolith.

## Writing Components

-   Use Typescript and add descriptions for the props.
    _Example:_
    
    ```jsx
    export interface GridProps {
        /** Child elements. */
        children?: any;
        /** CSS class names. */
        className?: string;
        /** Ref to the component. */
        ref?: Ref<HTMLElement>;
        /** Customize the tag element (defaults to 'div'). */
        tag?: ElementType;
    }
    ``` 

-   Don't use default exports.
    _Example:_

    -   Prefer this:

        ```jsx
        export const Grid: FunctionComponent<GridProps>
        ```

    -   Avoid this:

        ```jsx
        export default Grid;
        ```

-  Prefer composition over props. It's not always obvious when a value should be a prop or child element. Do your best to make the component easy to use, and allow the users to create their own level of abstraction.
    _Example:_

    -   Prefer this:

        ```html
        <Button>Button Label</Button>
        ```

    -   Avoid this:

        ```html
        <Button label="Button Label" />
        ```

-   Component names must be written in CamelCase.
-   Component must have its name as a `class` on the root element, written in kebab-case and prefixed with `axiom`.
    _Example: The component `Grid` will have the class `axiom-grid`._

-   Make it possible for teams to add customizations when necessary without being blocked:

    -   Accept an optional `className`. The string should be appended to the components existing class.
        _Example: `<Grid className="approve"...>` renders `class="axiom-grid approve"`._
    -   Pass down any extra props to the root element.
        _Example: `<Grid ohHello="but why?"/>` should pass on the `ohHello` prop to the underlying component.

        ```jsx
        // This example demonstrates how to support customization.

        interface GridProps {
            /** CSS class names. */
            className?: string;
        }

        export const Grid: FunctionComponent<GridProps> = forwardRef(
            ({ className, ...props}, ref: Ref<HTMLElement>) => (
                <div className={classNames("axiom-grid", className)} {...props} />
            )
        );
        ```
-   Add Storybook stories to demonstrate component functionality.

## Using Chromatic Visual Regression Testing (VRT)

Each pull request will automatically trigger a build in Chromatic to visually test every Storybook story. After the checks finish running on your PR, be sure to address any discrepancies in Chromatic, either by accepting the new snapshot (thus making it the new baseline) or denying the change and fixing any issues in your PR. For more information, refer to the [Chromatic documentation](https://docs.chromaticqa.com/).

### Disabling Chromatic for a Story

Sometimes you don't want Chromatic to run on a particular story, either because you are expecting changes or the story is repetitive. In this case, you can use a parameter on the story to disable chromatic:

```
stories.addParameters({ chromatic: { disable: true } });
```

### VRT for Dates

To prevent stories from failing VRT tests due to system time being used, consider setting the time in the story itself or disabling chromatic for that story if manually setting the time is not applicable.

## Creating a Release

Both UI Engineers and the Frontend team have permission to release Axiom via `yarn axiom version --xxxxx`:

1. Get latest code: `git checkout devel && git pull && git checkout master && git pull`
2. Merge your changes: `git merge devel`
3. Build static Storybook documentation for this release: `yarn axiom build-storybook`
4. Add a new header to `CHANGELOG.md` under “Unreleased” with the [new version number](https://medium.com/design-optimizely/how-to-version-your-ui-library-1c7a1b7ee23a):

   ```md
   ## Unreleased

   ## 31.0.0 - 2018-04-13

   - [Release] Added a cool breaking change. (#999)
   ```

5. Commit to master: `git add . && git commit -a -m 'Prep for new release version x.y.z'`
   - You'll be committing the CHANGELOG file and the new Storybook iframe.html and bundle
6. Run one of these depending on the highest importance issue this release:
   - `[Patch]` changes: `yarn axiom version --patch`
   - `[Feature]` changes: `yarn axiom version --minor`
   - `[Release]` changes: `yarn axiom version --major`
7. [Create a new release on GitHub](https://github.com/optimizely/axiom/releases/new):
   - Select the new tag version
   - Leave “Release title” blank
   - Paste in new release contributions from the `CHANGELOG.md` release notes into the Description field
   - Click Publish Release
8. Check for updated package on NPM
   - Check [NPM](https://www.npmjs.com/package/optimizely-oui) for the updated release
   - If it seems to stall, check [travis](https://travis-ci.com/github/optimizely/axiom/builds) and make sure no errors occurred
9. You're done :sunglasses: